package algorithm;
import java.util.*;

import main.MCSP;
import instance.*;
public class Solver {
	Instance inst;
	SampleGraph sg;
	public Solver(Instance inst){
		this.inst = inst;
	}
	
	public boolean solve(){
		if (inst.getK()<0) return false;
		sg = new SampleGraph(this.inst);
		if (sg.isNoInstance) {
			MCSP.print("Too many black edges");
			return false;
		}
		if (sg.hasBlackParallel()){
			MCSP.print("No because of parallel black edges");
			//return false;
		}
		MCSP.print("No parallel black edges");
		if (inst.getK()-sg.blackEdges.size()<0){
			MCSP.print("No because too many black edges");
			return false;
		}
		Marker branchMarker = sg.findIsolated();
		if (branchMarker!=null){
			MCSP.print("Found isolated marker "+branchMarker.tag());
			//branch on this marker
			return branch(branchMarker);
		}		
		//check for odd paths
		Set<Marker> branchMarkers = sg.findOddPath();
		if (branchMarkers!=null) return branch(branchMarkers);
		//if no odd paths		
		else {
			MCSP.print("Only Cycles and Even Paths");
			this.finishSolution();
			MCSP.storeSolution(inst.getTempSol());
			return true;
		}
	}
	
	private boolean branch(Set<Marker> branchMarkers){
		MCSP.print("Branching on Marker Set");
		for (Marker m: branchMarkers){
			MCSP.print("Branching on Marker:" +m.tag());
			if (branch(m)) return true;
		}
		return false;
	}
	
	private boolean branch(Marker branchMarker){
		Set<Marker> oldMatches = branchMarker.getMatches();
		for (Marker matchMarker : oldMatches) {
			MCSP.print("Fixing edge to "+matchMarker.tag());
			Set<Marker> matchMarkerMatches =  matchMarker.getMatches();
				
			//make Markers fixed in branches
			Set<Marker> bNewMatch =  new HashSet<Marker>();
			bNewMatch.add(matchMarker);
			Set<Marker> mNewMatch =  new HashSet<Marker>();
			mNewMatch.add(branchMarker);
			branchMarker.setMatches(bNewMatch);
			matchMarker.setMatches(mNewMatch);
			Marker g1Marker = inst.getFromGenome1(branchMarker,matchMarker);
			if (!inst.checkParallel(g1Marker)){
				//copy instance
				Instance newInst = InstanceFactory.copyInstance(inst);
				newInst.makeMatchesConsistent();
				if (newInst.solve(inst.getK())) return true;
			}
			else {
				MCSP.print("Aborted branch (Would give parallel black)");
			}
			//restore matchMarkers Matches
			matchMarker.setMatches(matchMarkerMatches);
		}
		//restore branchMarkers Matches
		branchMarker.setMatches(oldMatches);
		return false;
	}
	
	//add all black matches as fixes, all green edges of cycles and green even paths, 
	//and all red edges of red even paths
	private void finishSolution(){
		sg.addFixes();		
	}
	
}
