package instance;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import utilities.RandomGenerator;

import main.MCSP;

public class InstanceFactory {

	public static Instance createInstanceFromRandomBlocks(int n, int maxD, int numOfGeneFamilies, int k, int totalNoise) throws Exception{
		RandomGenerator G = new RandomGenerator(n, numOfGeneFamilies, maxD, k, totalNoise);
		return createInstanceFromArrays(G.genome0, G.genome1, G.sgn0, G.sgn1);
	}
	
	public static Instance createInstanceFromRandomBlocksSimple(int n,  int d, int k){
		if (k<1) k=1;
		if (n<1) n=1;
		if (d<2) d=2;
		RandomGenerator G;
		try {
			int families=(int)(3*n/d);
			if (families<2) families =2;
			G = new RandomGenerator(n, families, d, k, (int)(n/10));
		} catch (Exception e) {			
			throw new Error(e);
		}
		return createInstanceFromArrays(G.genome0, G.genome1, G.sgn0, G.sgn1);
	}
	
	public static Instance createInstanceFromArrays(int[] g1, int[] g2, int[] s1, int[] s2){
		Instance inst = new Instance();
		Genome genome1 = inst.getGenome1();		
		Genome genome2 = inst.getGenome2();
		for (int i = 0; i < g1.length; i++) {
			genome1.addMarker(new Marker(Marker.getNewId(), s1[i], i, g1[i],genome1));			
		}
		for (int i = 0; i < g2.length; i++) {
			genome2.addMarker(new Marker(Marker.getNewId(), s2[i], i, g2[i],genome2));			
		}
		for (Iterator<Marker> it1 = genome1.iterator(); it1.hasNext();) {
			Marker m1 = it1.next();
			for (Iterator<Marker> it2 = genome2.iterator(); it2.hasNext();) {
				Marker m2 = it2.next();
					if (m1.name.equals(m2.name)){
						m1.addMatch(m2);
						m2.addMatch(m1);
					}
			}
		}
		genome1.removeNonMatched();
		genome2.removeNonMatched();
		genome1.testGenome(genome2);
		genome2.testGenome(genome1);
		return inst;
	}
	
	public static Instance createInstancefromFileName(String fileName){
		Instance inst = new Instance();
		Genome genome1 = inst.getGenome1();		
		Genome genome2 = inst.getGenome2();
				
		try{
			BufferedReader inFile = new BufferedReader(new FileReader(fileName));
			String line= inFile.readLine();
			while (!line.startsWith("Printing")){
				line = inFile.readLine();
			}
			initialize(genome1, genome2, inFile);
			genome1.testGenome(genome2);
			genome2.testGenome(genome1);
			//inst.print();
		}
		catch (Exception e) {
			System.out.println(e.getMessage());
			
		}
		return inst;
	}
	
	public static void createGenomePointers(Genome g1, Genome g2){
		if (MCSP.verbose) System.out.println("Creating Gene Pointers");
		for (Iterator<Marker> it = g1.geneList().iterator(); it.hasNext();){
			Marker gene1 = it.next();
			for (Iterator<Integer> itid = gene1.matchIds.iterator(); itid.hasNext();){
				gene1.addMatch(g2.getMarkerById(itid.next()));
			}
		}
	}
	
	public static void readGenome(Genome genome, BufferedReader inFile) throws Exception{
		String line = inFile.readLine();		
		while (!line.startsWith("****")){
			line = inFile.readLine();
		}
		line = inFile.readLine();
		while (line.startsWith("id")){
			String description = inFile.readLine();
			String currentLine = inFile.readLine();
			Set<String> matchLines = new HashSet<String>();
			while (!currentLine.startsWith("id")&&!currentLine.startsWith("Genome")&&!currentLine.startsWith("Crea")){
				matchLines.add(currentLine);
				currentLine = inFile.readLine();					
			}				
			Marker gene = new Marker (line,description,matchLines,genome);
			genome.addMarker(gene);
			//gene.print();
			line = currentLine;
		}
	}
	
	private static void initialize(Genome g1, Genome g2, BufferedReader inFile) throws Exception {
		readGenome(g1,inFile);
		readGenome(g2,inFile);
		createGenomePointers(g1, g2);
		createGenomePointers(g2, g1);
		g1.makePointersTransitive(g2);
		g1.removeNonMatched();
		g2.removeNonMatched();
	}
	
	public static Instance copyInstance(Instance inst){
		Instance result = new Instance();
		Genome newGenome1 = result.getGenome1();
		Genome newGenome2 = result.getGenome2();		
		Genome oldGenome1 = inst.getGenome1();
		Genome oldGenome2 = inst.getGenome2();
		for (Iterator<Marker> it = oldGenome1.iterator(); it.hasNext();) {
			Marker current = it.next();
			newGenome1.addMarker(new Marker(current,newGenome1));
			
		}
		for (Iterator<Marker> it = oldGenome2.iterator(); it.hasNext();) {
			Marker current = it.next();
			newGenome2.addMarker(new Marker(current,newGenome2));			
		}		
		createGenomePointers(newGenome1, newGenome2);
		createGenomePointers(newGenome2, newGenome1);
		result.setTempSol(inst.getTempSol().copyTempSol());
		return result;
	}
	
}
