package instance;
import java.util.*;

public class Marker {
	private static int nextId=0;
	
	Integer id;
	private Integer chromosome;
	Integer sign;
	Integer startPosition;
	Marker nextFixed;
	public final String name;
	private Set<Marker> matches;
	Set<Integer> matchIds;
	
	public static int getNewId(){
		nextId++;
		return nextId-1;
	}
	Marker(int id, int sign, int startPosition, int name,Genome g){
		this.id = id;
		this.chromosome = 1;
		this.name = Integer.toString(name);
		this.startPosition = startPosition;
		this.sign = sign;
		this.matches = new HashSet<Marker>();
		this.matchIds = new HashSet<Integer>();	
		g.idGeneMap.put(id, this);
	}
	
	Marker(Marker oldMarker, Genome g){
		this.id = oldMarker.id;
		this.chromosome = oldMarker.chromosome;
		this.name = oldMarker.name;
		this.startPosition = oldMarker.startPosition;
		this.sign = oldMarker.sign;
		this.matches = new HashSet<Marker>();
		this.matchIds = new HashSet<Integer>();		
		for (Marker m : oldMarker.getMatches()) {
			this.matchIds.add(m.id);
		}
		g.idGeneMap.put(this.id, this);
	}
	
	Marker(String idLine,String descriptionLine, Set<String> matchLines,Genome g){
		this.matchIds = new HashSet<Integer>();
		this.setMatches(new  HashSet<Marker>());
		//Get id
		this.id =  new Integer (idLine.substring(3, idLine.length()-3));
		String[] description = descriptionLine.split("\t");
		this.name = description[1];
		this.setChromosome(new Integer(description[2]));
		this.startPosition = new Integer(description[3]);
		this.sign = new Integer(description[4]); 
		Integer match = new Integer (description[5]);
		if (match>0){
			matchIds.add(match);			
		}
		else {
			for (Iterator<String> it = matchLines.iterator();it.hasNext();){
				matchIds.add(new Integer(it.next().split("\t")[0]));
			}
		}
		g.idGeneMap.put(id, this);
	}
	
	public boolean isFixed(){
		if (getMatches().size()==1&&this.getMatch().getMatches().size()==1){
			return true;
		}
		else {
			return false;
		}
	}
	
	
	
	void print(){
		System.out.println("Tag = "+ this.tag());
		//System.out.println("Accession Number = "+ this.name);
		//System.out.println("Chromosome = "+ this.getChromosome());
		//System.out.println("Sign = "+ this.sign);
		//System.out.println("Start = "+ this.startPosition);
		if (this.isFixed()){
			System.out.println("Fixed with "+ this.getMatch().tag());
		}
		else {
			System.out.println("Not fixed with possibilities:");
			for (Iterator<Integer> it = matchIds.iterator(); it.hasNext();) {
				System.out.println("Possible: "+ it.next());
			}
		}
		
	}
	
	public Integer getId(){
		return this.id.intValue();
	}
	
	public void addMatch(Marker g){
		this.getMatches().add(g);
	}
	
	public Marker getMatch(){
		if(this.getMatches().isEmpty()) return null;
		else return this.getMatches().iterator().next();
	}
	
	public boolean isForwardMatch(Marker g){
		if (this.getMatches().contains(g)&&this.sign.intValue()==g.sign.intValue()) return true;
		else return false;
	}
	
	public boolean isBackwardMatch(Marker g){
		if (this.getMatches().contains(g)&&this.sign!=g.sign) return true;
		else return false;
	}
	
	public boolean isMatch(Marker m2){
		return this.getMatches().contains(m2);
	}
	
	public int matchNum(){
		return this.getMatches().size();
	}

	public int getChromosome() {
		return chromosome.intValue();
	}

	public void setChromosome(Integer chromosome) {
		this.chromosome = chromosome;
	}

	public String tag(){
		return this.name+"."+this.id;
	}
	
	public Set<Marker> getMatches() {
		return matches;
	}

	public void setMatches(Set<Marker> matches) {
		this.matches = matches;
	}
	
	public boolean isRare(){
		Marker match = this.getMatch();
		if (match==null) return false;
		else return (match.matchNum()<=this.matchNum());
	}
	
}
