package main;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.Locale;

import instance.Instance;
import instance.InstanceFactory;

public class RunningTimeSynthetic {
	int n;
	int k;
	int d;
	int geneFamNum;
	int noise;
	
	//parameters for array of tests :	
	final int repetitionNum = 150;
	//  timeout: stops after timeOut for each cell; shows the average over reduced data if at least one try is complete
	final int timeOutPerPerformSeconds = 300; 
	// tested values of n, k and d
	final int[] ns = { 200, 1000,  2000 };
	final int[] ks = { 25, 50, 75, 100, 125, 150, 175, 200 };
	final int[] ds = { 2, 5, 8 };

	
	boolean stopByTimeOut;// whether the last call to perform was stopped by a timeout.
	
	
	float dAvg = 0;

	public RunningTimeSynthetic(int n, int d, int numOfGeneFamilies, int k,
			int noise) {
		this.n = n;
		this.k = k;
		this.geneFamNum = numOfGeneFamilies;
		this.d = d;
		this.noise = noise;
	}

	public RunningTimeSynthetic(int n, int d, int numOfGeneFamilies, int k) {
		this.n = n;
		this.k = k;
		this.geneFamNum = numOfGeneFamilies;
		this.d = d;
		this.noise = (int) (n / 10);
	}
	
	private void append(ArrayList<String> l, int id, String s) {
		l.set(id,l.get(id)+s);
	}
	
	
	public void MakeTable() {
		
		ArrayList<String> LatexRows=new ArrayList<String>();
		LatexRows.add("\\begin{tabular}{|c|");
		LatexRows.add("}\n \\hline \n$d=$");
		LatexRows.add("\\\\ \\hline \n $n=$");		
		boolean hline=true;
		for (int k: ks) {		
			if (hline) 
			   LatexRows.add("\\\\ \\hline\n$k="+String.valueOf(k)+"$");
			else
			   LatexRows.add("\\\\  \n$k="+String.valueOf(k)+"$");
			hline=false;
		}
		LatexRows.add("\\hline \n \\end{tabular}");		
		DecimalFormat latexFormat = new DecimalFormat("#0.00", new DecimalFormatSymbols(Locale.ENGLISH));		
		for (int d:ds) {			
			this.d = d;
			append(LatexRows,1," & \\multicolumn{"+ns.length+"}{c|}{"+d+"}");			
			for (int n : ns) {
				this.n = n;
				append(LatexRows,0,"c");
				append(LatexRows,2," & "+n);
				
			    System.out.println("-- d="+d+", n="+n+" --");
			    System.out.println("k\ttime\tstoppedAtTimeOut");			    
				for (int ik=0; ik<ks.length; ik++) {
					k = ks[ik];
					double time = perform(0, 0, true);
					if (stopByTimeOut) {
							System.out.println(k+"\t" + time + "\tyes");
						if (time > 0) {
							append(LatexRows,ik+3," & "+latexFormat.format(time)+"$^*$");
						} else {
							append(LatexRows,ik+3," &--$^*$");
						}
					} else {
						System.out.println(k+"\t" + time+"\t");
						append(LatexRows,ik+3," &"+latexFormat.format(time));
					}
				}
			}
			append(LatexRows,0,"|");
		}
		System.out.println("Latex:\n\n");
  
		for (String row:LatexRows) {
			System.out.print(row);			  	
		}		

		System.out.println("");
		

	}

	public void varynoise() {
		System.out
				.println("n\td\tk\tgeneFam\tnoise\trepetitions\tvarying noise");
		System.out.println(n + "\t" + d + "\t" + k + "\t" + geneFamNum + "\t"
				+ noise + "\t" + repetitionNum);
		for (noise = 0; 3 * noise < n; noise += 3) {
			double runningTime = perform(0, 0, false);
			System.out.println(noise + "\t" + runningTime);
			System.out.flush();
		}
	}

	public void varyNumOfFamilies(int start) {
		System.out
				.println("n\td\tk\tgeneFam\tnoise\trepetitions\tvarying geneFam");
		System.out.println(n + "\t" + d + "\t" + k + "\t" + geneFamNum + "\t"
				+ noise + "\t" + repetitionNum);
		// System.out.println("n\td\tk\tgeneFam\tnoise\tvarying k");
		// System.out.println(n+"\t"+d+"\t"+k+"\t auto \t auto");
		for (geneFamNum = start; geneFamNum < 3 * n; geneFamNum = (int) (geneFamNum * 1.1 + 1)) {
			dAvg = 0;
			d = (int) (2 * n / geneFamNum);
			if (d < 2)
				d = 2;
			double runningTime = perform(0, 0, false);
			dAvg = dAvg / repetitionNum;
			System.out.println(geneFamNum + "\t" + d + "\t" + runningTime
					+ "\t" + dAvg);
			System.out.flush();
		}
	}

	public void varyk(int range) {
		System.out.println("n\td\tk\tgeneFam\tnoise\tvarying k");
		System.out.println(n + "\t" + d + "\t" + k + "\t auto \t auto");
		for (int i = -range; i <= range; i++) {
			dAvg = 0;
			double runningTime = perform(0, 2*i, true);
			dAvg = dAvg / repetitionNum;
			System.out.println(k + 2*i + "\t" + runningTime + "\t" + dAvg);
			System.out.flush();
		}
	}

	public void varyn(int range) {
		// createInstanceFromRandomBlocks(int n, int maxD, int
		// numOfGeneFamilies, int k, int totalNoise)
		System.out.println("n\td\tk\tgeneFam\tnoise\tvarying n");
		System.out.println(n + "\t" + d + "\t" + k + "\t auto \t auto");
		for (int i = -range; i <= range; i += 10) {
			dAvg = 0;
			double runningTime = perform(i, 0, true);
			dAvg = dAvg / repetitionNum;
			System.out.println(n + i + "\t" + runningTime + "\t" + dAvg);
			System.out.flush();
		}
	}

	public double perform(int nOff, int kOff, boolean autoValues) {

		try {
			stopByTimeOut = false;
			long overAllTime = 0;
			double averageTime = 0;
			for (int i = 0; i < repetitionNum && !stopByTimeOut; i++) {
				Instance inst;
				if (autoValues)
					inst = InstanceFactory
							.createInstanceFromRandomBlocksSimple(n + nOff, d,
									k + kOff);
				else
					inst = InstanceFactory.createInstanceFromRandomBlocks(n
							+ nOff, d, geneFamNum, k + kOff, noise);
				dAvg += inst.getGenome1().dAvg();
				MCSP.storeInstance(inst);
				long start = System.currentTimeMillis();
				long timeOut = start + timeOutPerPerformSeconds * 1000;
				boolean solved = false;
				for (int k2 = 1; k2 <= k + kOff; k2++) {
					if (System.currentTimeMillis() > timeOut) {
						stopByTimeOut = true;
						break;
					}
					if (inst.solve(k2)) {
						overAllTime += System.currentTimeMillis() - start;
						solved = true;
						break;
					}
				}
				if (!solved && !stopByTimeOut)
					System.err.println("Did not solve some Yes-Instance");
				if (solved)
					MCSP.checkCorrectness(true);
				if (!stopByTimeOut)
					averageTime = (double) overAllTime/ ((double) repetitionNum * 1000);
			}
			return (averageTime);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			return -1;
		}
	}
}
